/// <reference types="node" resolution-mode="require"/>
import http from 'node:http';
import type { Composer } from './compose.js';
import type { BaseContext } from './middleware.js';
import type QueryClient from './query/client.js';
export type CreateServer<T extends unknown[], InputContext extends BaseContext, OutputContext extends BaseContext> = {
    /** Port to listen, default to 9001. */
    port?: number;
    /** The middleware composer to execute. */
    composer?: Composer<T, InputContext, OutputContext>;
    /** An array of query clients. The server will automatically init these clients in `listen()` and end these clients when server closing. */
    queryClients?: QueryClient[];
    /** Use 'none' to turn off the log. */
    logLevel?: 'none' | 'info';
    /**
     * If enabled, the server will add a 16-character request ID to the following places:
     *
     * - The context object (`context.requestId`)
     * - The response header
     * - The console log
     *
     * The request ID will be added to the response header with the name specified in `headerName`. Default to `x-request-id`.
     */
    requestId?: {
        enabled: true;
        /**
         * The header name to add the request ID.
         *
         * @default 'x-request-id'
         */
        headerName?: string;
    };
};
export declare const handleError: (response: http.ServerResponse, error: unknown) => Promise<void>;
/**
 * Create a new withtyped server with the given config.
 *
 * @param config The config object.
 * @returns An object including the server and its utilities.
 */
export default function createServer<T extends unknown[], OutputContext extends BaseContext>(config?: CreateServer<T, BaseContext, OutputContext>): {
    server: http.Server<typeof http.IncomingMessage, typeof http.ServerResponse>;
    /** Shut down all query clients and the server. */
    close: () => Promise<void>;
    /** Start all query clients and the server in order. */
    listen: (callback?: ((port: number) => void) | undefined) => Promise<void>;
};
export * from './types.js';
export { default as RequestError } from './errors/RequestError.js';
export * from './middleware/index.js';
export * from './openapi/openapi-types.js';
export { default as Router } from './router/index.js';
export * from './router/index.js';
export { default as compose } from './compose.js';
export * from './middleware.js';
export { RequestMethod } from '@withtyped/shared';
export { default as ModelClient } from './model-client/index.js';
export * from './model-client/index.js';
export { default as DatabaseInitializer } from './database-initializer/index.js';
export * from './query/index.js';
export * from './identifiable/index.js';
