import { contentTypes, log, RequestMethod } from '@withtyped/shared';
import RequestError from '../errors/RequestError.js';
const tryParse = (body) => {
    try {
        // `body` is not `any`, but `JSON.parse()` returns `any`. :shrug:
        // eslint-disable-next-line @typescript-eslint/no-unsafe-return
        return JSON.parse(body);
    }
    catch (error) {
        log.debug('failed to parse JSON string in `withBody()`', error);
    }
};
export default function withBody() {
    return async (context, next, { request }) => {
        if (!context.request.method ||
            [RequestMethod.GET, RequestMethod.OPTIONS, RequestMethod.HEAD].includes(context.request.method)) {
            // eslint-disable-next-line no-restricted-syntax
            return next(context); // No need to spread since nothing needs to change
        }
        const readBody = async () => new Promise((resolve, reject) => {
            const body = [];
            // eslint-disable-next-line @silverhand/fp/no-mutating-methods
            const pushToBody = (chunk) => body.push(chunk);
            request
                .on('data', pushToBody)
                .once('end', () => {
                request.removeListener('data', pushToBody);
                resolve(Buffer.concat(body));
            })
                .once('error', (error) => {
                request.removeListener('data', pushToBody);
                reject(error);
            });
        });
        const raw = await readBody();
        const rawString = raw.toString();
        if (!rawString) {
            // eslint-disable-next-line no-restricted-syntax
            return next(context);
        }
        if (context.request.headers['content-type']
            ?.split(';')
            .every((value) => value.trim() !== contentTypes.json)) {
            throw new RequestError(`Unexpected \`content-type\` header, only accept "${contentTypes.json}"`, 400);
        }
        return next({
            ...context,
            request: { ...context.request, body: tryParse(rawString), bodyRaw: raw },
        });
    };
}
