import { z } from 'zod';
import { type IdentifiableModel } from '../identifiable/index.js';
import type { Entity, EntityHasDefaultKeys, ModelCreateType, ModelExtendConfig, ModelExtendConfigWithDefault, ModelParseReturnType, ModelParseType, ModelPatchType, NormalizedBody, RawParserConfig, SplitRawColumns, TableName } from './types.js';
export type ModelZodObject<M> = z.ZodObject<{
    [Key in keyof M]-?: undefined extends M[Key] ? z.ZodOptional<z.ZodType<Exclude<M[Key], undefined>>> : z.ZodType<M[Key]>;
}>;
export default class Model<Table extends string = '', ModelType extends Record<string, unknown> = {}, DefaultKeys extends string = never, ReadonlyKeys extends string = never> {
    readonly raw: string;
    readonly extendedConfigs: Record<string, ModelExtendConfig<unknown>>;
    readonly excludedKeys: string[];
    readonly schema: string | undefined;
    /**
     * Create a new model instance according to the raw query of creating a table.
     *
     * @param raw Raw query of creating a table. An ending semicolon `;` is required.
     * @param schema Schema name of the table.
     *
     * @example
     * ```ts
     * const model = Model.create(`
     *   create table foo (id varchar(32) primary key not null, bar bigint);
     * `);
     *
     * type Model = InferModelType<typeof model>; // { id: string; bar: number; }
     * ```
     *
     * And it is case-insensitive:
     *
     * @example
     * ```ts
     * const model = Model.create(`
     *   cREaTe taBle forms (
     *     id VARCHAR(32) not null,
     *     remote_address varchar(128),
     *     headers jsonb not null,
     *     data jsonb
     *   );
     * `);
     *
     * type Model = InferModelType<typeof model>;
     * // {
     * //   id: string;
     * //   remoteAddress: string;
     * //   headers: JsonObject | JsonArray;
     * //   data: JsonObject | JsonArray | null;
     * // }
     * ```
     */
    static create: <Raw extends string>(raw: Raw, schema?: string) => Model<TableName<Raw>, Entity<SplitRawColumns<NormalizedBody<Raw>>>, EntityHasDefaultKeys<SplitRawColumns<NormalizedBody<Raw>>>, never>;
    /** Alias of {@link getGuard()}. */
    readonly guard: {
        <ForType extends ModelParseType = "model">(): ModelZodObject<ModelParseReturnType<ModelType, DefaultKeys, ReadonlyKeys>[ForType]>;
        <ForType_1 extends ModelParseType>(forType: ForType_1): ModelZodObject<ModelParseReturnType<ModelType, DefaultKeys, ReadonlyKeys>[ForType_1]>;
    };
    readonly tableName: Table;
    readonly rawConfigs: Readonly<Record<string & keyof ModelType, RawParserConfig>>;
    readonly rawKeys: Readonly<Record<keyof ModelType, string>>;
    protected readonly excludedKeySet: Set<string>;
    /** @internal Use {@link create()} instead. */
    constructor(raw: string, extendedConfigs: Record<string, ModelExtendConfig<unknown>>, excludedKeys: string[], schema: string | undefined);
    /**
     * Get the identifiable value of the model. It can be used in sql tag to reference the
     * model and its columns.
     *
     * @example
     * ```ts
     * const model = Model.create(`
     *  create table foo (id varchar(32) primary key not null, bar bigint);
     * `);
     * const { id, bar } = model.identifiable;
     *
     * sql`select ${id}, ${bar} from ${model.identifiable}`;
     * // select "foo"."id", "foo"."bar" from "foo"
     * ```
     *
     * It also works with a schema:
     *
     * @example
     * ```ts
     * const model = Model.create(`
     *   create table foo (id varchar(32) primary key not null, bar bigint);
     * `, 'baz');
     * const { id, bar } = model.identifiable;
     *
     * sql`select ${id}, ${bar} from ${model.identifiable}`;
     * // select "baz"."foo"."id", "baz"."foo"."bar" from "baz"."foo"
     * ```
     *
     * To update a column, use the `update` property since update queries are not allowed
     * to reference the table name:
     *
     * @example
     * ```ts
     * const model = Model.create(`
     *  create table foo (id varchar(32) primary key not null, bar bigint);
     * `);
     * const { id, bar } = model.identifiable;
     *
     * sql`update ${model.identifiable} set ${bar.update} = ${bar}`;
     * // update "foo" set "bar" = $1
     * ```
     *
     * @see {@link IdentifiableModel} For more details.
     */
    get identifiable(): Readonly<IdentifiableModel<ModelType>>;
    /**
     * Use a customized parser (guard) for the given key. Affects all model parsers.
     *
     * NOTE: Has no effect to sql helpers.
     *
     * @see {@link convertConfigToZod} For how it affects model guards.
     */
    extend<Key extends string & keyof ModelType, Type>(key: Key, parser: z.ZodType<Type>): Model<Table, {
        [key in keyof ModelType]: key extends Key ? Type : ModelType[key];
    }, DefaultKeys, ReadonlyKeys>;
    /**
     * Use a programmatic default value for the given key with an optional customized parser (guard).
     * The customized parser affects all model parsers.
     *
     * NOTE: Has no effect to sql helpers.
     *
     * @see {@link convertConfigToZod} For how it affects model guards.
     */
    extend<Key extends string & keyof ModelType, Type>(key: Key, config: ModelExtendConfigWithDefault<Type>): Model<Table, {
        [key in keyof ModelType]: key extends Key ? Type : ModelType[key];
    }, DefaultKeys | Key, ReadonlyKeys>;
    /**
     * Mark a key with default value readonly.
     *
     * NOTE: Has no effect to sql helpers.
     *
     * @see {@link convertConfigToZod} For how it affects model guards.
     */
    extend<Key extends string & DefaultKeys>(key: Key, config: Pick<ModelExtendConfigWithDefault<unknown, true>, 'readonly'>): Model<Table, ModelType, DefaultKeys, ReadonlyKeys | Key>;
    /**
     * Use a programmatic default value for the given key with an optional customized parser (guard).
     * The customized parser affects all model parsers.
     *
     * NOTE: Has no effect to sql helpers.
     *
     * @see {@link convertConfigToZod} For how it affects model guards.
     */
    extend<Key extends string & keyof ModelType, Type, RO extends boolean>(key: Key, config: ModelExtendConfigWithDefault<Type, true>): Model<Table, {
        [key in keyof ModelType]: key extends Key ? Type : ModelType[key];
    }, DefaultKeys | Key, ReadonlyKeys | Key>;
    /** Exclude a key from the model. */
    exclude<Key extends string & keyof ModelType>(key: Key): Model<Table, {
        [key in keyof ModelType as key extends Key ? never : key]: ModelType[key];
    }, Exclude<DefaultKeys, Key>, Exclude<ReadonlyKeys, Key>>;
    /**
     * Get the 'model' zod guard of the current model.
     *
     * @see {@link convertConfigToZod} For details of model guards.
     */
    getGuard<ForType extends ModelParseType = 'model'>(): ModelZodObject<ModelParseReturnType<ModelType, DefaultKeys, ReadonlyKeys>[ForType]>;
    /**
     * Get the related zod guard of the current model.
     *
     * @param forType One of 'model' (default), 'create', or 'patch'.
     * @see {@link InferModelCreate} For what will be transformed for a create type guard.
     * @see {@link InferModelPatch} For what will be transformed for a patch type guard.
     * @see {@link convertConfigToZod} For details of model guards.
     */
    getGuard<ForType extends ModelParseType>(forType: ForType): ModelZodObject<ModelParseReturnType<ModelType, DefaultKeys, ReadonlyKeys>[ForType]>;
    /**
     * Parse the given data using the 'model' guard. Kebab-case and snake_case fields will be camelCased.
     *
     * @param data The data to parse.
     * @see {@link convertConfigToZod} For details of model guards.
     * @throws `ZodError` when parse failed.
     */
    parse(data: unknown): ModelParseReturnType<ModelType, DefaultKeys, ReadonlyKeys>['model'];
    /**
     * Parse the given data using a designated model guard. Kebab-case and snake_case fields will be camelCased.
     *
     * @param data The data to parse.
     * @param forType Use which guard to parse. One of 'model' (default), 'create', or 'patch'.
     * @see {@link InferModelCreate} For what will be transformed for a create type guard.
     * @see {@link InferModelPatch} For what will be transformed for a patch type guard.
     * @see {@link convertConfigToZod} For details of model guards.
     * @throws `ZodError` when parse failed.
     */
    parse<ForType extends ModelParseType>(data: unknown, forType: ForType): ModelParseReturnType<ModelType, DefaultKeys, ReadonlyKeys>[ForType];
}
export declare const createModel: <Raw extends string>(raw: Raw, schema?: string) => Model<TableName<Raw>, Entity<SplitRawColumns<NormalizedBody<Raw>>>, EntityHasDefaultKeys<SplitRawColumns<NormalizedBody<Raw>>>, never>;
/** Infers the model type of a given model class. */
export type InferModel<M> = M extends Model<string, infer A> ? A : never;
/** Alias of {@link InferModel}. */
export type InferModelType<M> = InferModel<M>;
/**
 * Infers the model creation type of a given model class.
 *
 * The model creation type makes the original model fields with default value optional,
 * and forces readonly fields to be set as `undefined`.
 */
export type InferModelCreate<M> = M extends Model<string, infer ModelType, infer DefaultKeys, infer ReadonlyKeys> ? ModelCreateType<ModelType, DefaultKeys, ReadonlyKeys> : never;
/**
 * Infers the model patch type of a given model class.
 *
 * The model patch type makes all fields optional, except readonly fields are omitted.
 */
export type InferModelPatch<M> = M extends Model<string, infer ModelType, string, infer ReadonlyKeys> ? ModelPatchType<ModelType, ReadonlyKeys> : never;
export * from './types.js';
export * from './utils.js';
