import { z } from 'zod';
import { idSymbols, } from '../identifiable/index.js';
import { camelCaseKeys, parseRawConfigs, parseTableName } from './utils.js';
import { convertConfigToZod } from './utils.zod.js';
export default class Model {
    /**
     * Create a new model instance according to the raw query of creating a table.
     *
     * @param raw Raw query of creating a table. An ending semicolon `;` is required.
     * @param schema Schema name of the table.
     *
     * @example
     * ```ts
     * const model = Model.create(`
     *   create table foo (id varchar(32) primary key not null, bar bigint);
     * `);
     *
     * type Model = InferModelType<typeof model>; // { id: string; bar: number; }
     * ```
     *
     * And it is case-insensitive:
     *
     * @example
     * ```ts
     * const model = Model.create(`
     *   cREaTe taBle forms (
     *     id VARCHAR(32) not null,
     *     remote_address varchar(128),
     *     headers jsonb not null,
     *     data jsonb
     *   );
     * `);
     *
     * type Model = InferModelType<typeof model>;
     * // {
     * //   id: string;
     * //   remoteAddress: string;
     * //   headers: JsonObject | JsonArray;
     * //   data: JsonObject | JsonArray | null;
     * // }
     * ```
     */
    static { this.create = (raw, schema) => {
        return new Model(raw, Object.freeze({}), [], schema);
    }; }
    /** @internal Use {@link create()} instead. */
    constructor(raw, extendedConfigs, excludedKeys, schema) {
        this.raw = raw;
        this.extendedConfigs = extendedConfigs;
        this.excludedKeys = excludedKeys;
        this.schema = schema;
        /** Alias of {@link getGuard()}. */
        this.guard = this.getGuard;
        const tableName = parseTableName(raw);
        if (!tableName) {
            throw new TypeError('Table name not found in query');
        }
        // eslint-disable-next-line no-restricted-syntax
        this.tableName = tableName;
        this.rawConfigs = Object.freeze(parseRawConfigs(raw));
        this.excludedKeySet = new Set(this.excludedKeys);
        const entries = Object.entries(this.rawConfigs).filter(([key]) => !this.excludedKeySet.has(key));
        this.rawKeys = Object.freeze(
        // eslint-disable-next-line no-restricted-syntax
        Object.fromEntries(entries.map(([key, { rawKey }]) => [key, rawKey])));
    }
    /**
     * Get the identifiable value of the model. It can be used in sql tag to reference the
     * model and its columns.
     *
     * @example
     * ```ts
     * const model = Model.create(`
     *  create table foo (id varchar(32) primary key not null, bar bigint);
     * `);
     * const { id, bar } = model.identifiable;
     *
     * sql`select ${id}, ${bar} from ${model.identifiable}`;
     * // select "foo"."id", "foo"."bar" from "foo"
     * ```
     *
     * It also works with a schema:
     *
     * @example
     * ```ts
     * const model = Model.create(`
     *   create table foo (id varchar(32) primary key not null, bar bigint);
     * `, 'baz');
     * const { id, bar } = model.identifiable;
     *
     * sql`select ${id}, ${bar} from ${model.identifiable}`;
     * // select "baz"."foo"."id", "baz"."foo"."bar" from "baz"."foo"
     * ```
     *
     * To update a column, use the `update` property since update queries are not allowed
     * to reference the table name:
     *
     * @example
     * ```ts
     * const model = Model.create(`
     *  create table foo (id varchar(32) primary key not null, bar bigint);
     * `);
     * const { id, bar } = model.identifiable;
     *
     * sql`update ${model.identifiable} set ${bar.update} = ${bar}`;
     * // update "foo" set "bar" = $1
     * ```
     *
     * @see {@link IdentifiableModel} For more details.
     */
    get identifiable() {
        const metadata = Object.freeze({
            schema: this.schema,
            tableName: this.tableName,
        });
        return Object.freeze({
            [idSymbols.model]: metadata,
            // eslint-disable-next-line no-restricted-syntax
            ...Object.fromEntries(Object.entries(this.rawKeys).map(([key, rawKey]) => [
                key,
                {
                    [idSymbols.column]: true,
                    model: metadata,
                    name: rawKey,
                    get update() {
                        return Object.freeze({ [idSymbols.updateColumn]: true, name: rawKey });
                    },
                },
            ])),
        });
    }
    extend(key, config) {
        return new Model(this.raw, Object.freeze({
            ...this.extendedConfigs,
            [key]: {
                ...this.extendedConfigs[key],
                ...(config instanceof z.ZodType ? { parser: config } : config),
            },
        }), this.excludedKeys, this.schema);
    }
    /** Exclude a key from the model. */
    exclude(key) {
        return new Model(this.raw, this.extendedConfigs, this.excludedKeys.concat(key), this.schema);
    }
    getGuard(forType) {
        // eslint-disable-next-line @silverhand/fp/no-let
        let guard = z.object({});
        // Merge config first
        for (const [key, rawConfig] of Object.entries(this.rawConfigs)) {
            if (this.excludedKeySet.has(key) || key in guard.shape) {
                continue;
            }
            const parser = convertConfigToZod(rawConfig, this.extendedConfigs[key], forType ?? 'model');
            if (parser) {
                // eslint-disable-next-line @silverhand/fp/no-mutation
                guard = guard.extend({ [key]: parser });
            }
        }
        // eslint-disable-next-line no-restricted-syntax
        return guard;
    }
    parse(data, forType) {
        // eslint-disable-next-line no-restricted-syntax
        return this.getGuard(forType ?? 'model').parse(camelCaseKeys(data));
    }
}
export const createModel = Model.create;
export * from './types.js';
export * from './utils.js';
