import { log } from '@withtyped/shared';
import { createComposer } from '../compose.js';
import RequestError from '../errors/RequestError.js';
import { ModelClientError } from '../model-client/errors.js';
import { buildOpenApiJson } from './openapi.js';
import Route from './route/index.js';
import { matchRoute } from './utils.js';
export * from './types.js';
export const openApiRoute = '/openapi.json';
/**
 * WARNING: Don't use this function unless you know what you are doing.
 *
 * Compose an array of middleware functions into a single middleware function,
 * by filtering out all falsy values. This function does NOT check the type of
 * the input, so you should make sure the input is an array of middleware
 * functions, and manually specify the input and output context types.
 */
const composeArray = (...runs) => {
    return createComposer(runs.filter(Boolean));
};
export default class Router {
    constructor(prefix) {
        // Use the dumb way to init since it's easier to make the compiler happy
        this.get = this.buildRoute('get');
        this.post = this.buildRoute('post');
        this.put = this.buildRoute('put');
        this.patch = this.buildRoute('patch');
        this.delete = this.buildRoute('delete');
        this.copy = this.buildRoute('copy');
        this.head = this.buildRoute('head');
        this.options = this.buildRoute('options');
        this.routesMap = {};
        this.middlewareArray = [];
        if (prefix && (prefix.endsWith('/') || prefix.includes('//') || prefix.includes(':'))) {
            throw new TypeError('Router prefix must be normalized with no parameter (colon)');
        }
        this.prefix = prefix ?? '';
    }
    routes() {
        return async (originalContext, next, http) => {
            const { request } = originalContext;
            // TODO: Consider best match instead of first match
            const route = this.routesMap[request.method?.toLowerCase() ?? '']?.find((route) => matchRoute(route, request.url));
            if (!route) {
                return next(originalContext);
            }
            log.debug('matched route', this.prefix, route.path);
            const run = composeArray(...this.middlewareArray, route.runnable);
            try {
                await run(originalContext, async (context) => {
                    const responseGuard = route.guard.response;
                    if (!responseGuard && route.path !== openApiRoute && context.json !== undefined) {
                        throw new TypeError('Response guard is required when providing a response json.');
                    }
                    return next({
                        ...context,
                        json: responseGuard?.parse(context.json) ?? context.json,
                        status: context.status ?? (context.json ? 200 : 204),
                    });
                }, http);
            }
            catch (error) {
                // TODO: Consider some better approach to decouple ModelClientError with Router
                if (error instanceof ModelClientError && error.code === 'entity_not_found') {
                    throw new RequestError('Entity not found', 404);
                }
                throw error;
            }
        };
    }
    use(middleware) {
        if (Object.values(this.routesMap).length > 0) {
            throw new Error('Middleware must be added before adding routes');
        }
        // @ts-expect-error Force update for the new input context
        // eslint-disable-next-line @silverhand/fp/no-mutating-methods
        this.middlewareArray.push(middleware);
        // @ts-expect-error Force update for the new input context
        // eslint-disable-next-line no-restricted-syntax
        return this;
    }
    withOpenApi(parseSearch, parse, info) {
        return this.get(openApiRoute, {}, async (context, next) => {
            return next({
                ...context,
                json: buildOpenApiJson(this.routesMap, parseSearch, parse, info),
            });
        });
    }
    pack(another // Don't care another prefix since routes are all prefixed
    ) {
        // TODO: Consider to add the instance from another router to support middleware
        if (another.middlewareArray.length > 0) {
            throw new Error('Another router must not have middleware');
        }
        for (const [method, routes] of Object.entries(another.routesMap)) {
            this.routesMap[method] = (this.routesMap[method] ?? []).concat(
            // @ts-expect-error It's ok for another router to have a different input context since:
            // - We don't care the output contexts of routers.
            // - As long as the pre-input context compatible with the current one
            routes.map((instance) => instance.clone(this.prefix + instance.prefix)));
        }
        // Intended
        // eslint-disable-next-line no-restricted-syntax
        return this;
    }
    findRoute(method, path) {
        const url = new URL(String(path), 'https://fake-url');
        return this.routesMap[method]?.find((route) => matchRoute(route, url));
    }
    buildRoute(method) {
        // @ts-expect-error The function overload it too complex to make TypeScript happy
        // We'll make it right in the implementation
        return (path, guard, ...runs) => {
            this.routesMap[method] = (this.routesMap[method] ?? []).concat(new Route(this.prefix, path, guard, composeArray(...runs)));
            // eslint-disable-next-line no-restricted-syntax
            return this;
        };
    }
}
export const createRouter = (prefix) => (prefix ? new Router(prefix) : new Router()); // To make TypeScript happy
